from imagekit.specs import ImageSpec 
from imagekit import processors

from django.conf import settings

# thumbnail resize processor 
class ResizeThumb(processors.Resize): 
    width = settings.LOUPE_RESIZE_THUMB_WIDTH
    height = settings.LOUPE_RESIZE_THUMB_HEIGHT 
    crop = settings.LOUPE_RESIZE_THUMB_CROP

# display size resize processor
class ResizeDisplay(processors.Resize):
    width = settings.LOUPE_RESIZE_DISPLAY_WIDTH

# adjustment processor to enhance the image at small sizes 
class EnchanceThumb(processors.Adjustment): 
    contrast = 1.2 
    sharpness = 1.1 

# define our thumbnail spec 
class Thumbnail(ImageSpec): 
    access_as = 'thumbnail_image' 
    pre_cache = settings.LOUPE_PRE_CACHE_IMAGES 
    processors = [ResizeThumb, EnchanceThumb] 

# and our display spec
class Display(ImageSpec):
    access_as = 'display'
    pre_cache = settings.LOUPE_PRE_CACHE_IMAGES
    increment_count = settings.LOUPE_INCREMENT_COUNT
    processors = [ResizeDisplay]



from django.db import models
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from datetime import datetime
from django.db.models import permalink
from django.contrib.auth.models import User
from django_extensions.db.models import TitleSlugDescriptionModel
from django_extensions.db.fields import AutoSlugField

from imagekit.models import ImageModel

class Project(TitleSlugDescriptionModel):
    """
    Stores the projects that corkboards belong to.
    Only administrators should be able to set these up
    """

    user = models.ForeignKey(User)
    members = models.ManyToManyField(User, related_name="ProjectMember")
    created_on = models.DateTimeField(_('Created On'), default=datetime.now, editable=False)
    updated_on = models.DateTimeField(_('Updated On'), editable=False, null=True, blank=True)

    class Meta:
        ordering = ['-created_on',]

    def __unicode__(self):
        return '%s' % (self.title)
    
    def save(self, *args, **kwargs):
        self.updated_on = datetime.now()
        super(Project, self).save()

class Corkboard(TitleSlugDescriptionModel):
    """
    Stores the grouping of images to be reviewed by the project team
    Example.  For a home page mockup this group would contain all mockups
    for only the home page to reduce confusion when reviewing designs.
    """  
    user = models.ForeignKey(User)
    project = models.ForeignKey(Project)
    created_on = models.DateTimeField(_('Created On'), default=datetime.now, editable=False)
    updated_on = models.DateTimeField(_('Updated On'), editable=False, null=True, blank=True)
    
    class Meta:
        ordering = ['-created_on',]

    def __unicode__(self):
        return '%s' % (self.title)
    
    def save(self, *args, **kwargs):
        self.updated_on = datetime.now()
        super(Corkboard, self).save()

class Image(ImageModel):
    title = models.CharField(max_length=200)
    corkboard = models.ForeignKey(Corkboard)
    original_image = models.ImageField(upload_to='loupe')
    user = models.ForeignKey(User)
    num_views = models.PositiveIntegerField(_('Number of Views'), editable=False, default=0)
    created_on = models.DateTimeField(_('Created On'), default=datetime.now, editable=False)
    updated_on = models.DateTimeField(_('Updated On'), editable=False, null=True, blank=True)

    class IKOptions:
        # This inner class is where we define the ImageKit options for the model
        spec_module = 'loupe.specs'
        cache_dir = 'loupe'
        image_field = 'original_image'
        save_count_as = 'num_views'

    def __unicode__(self):
        return '%s' % (self.title)

    def save(self, *args, **kwargs):
        self.updated_on = datetime.now()
        super(Image, self).save()

class Note(models.Model):
    """
    Stores image notes for the jquery hover blocks that are placed
    over the images
    """
    user = models.ForeignKey(User)
    image = models.ForeignKey(Image)
    note = models.TextField(_('Note'))
    x1 = models.PositiveIntegerField(_('x1'), default=0)
    y1 = models.PositiveIntegerField(_('y1'), default=0)
    height = models.PositiveIntegerField(_('Height'), default=20)
    width = models.PositiveIntegerField(_('Width'), default=20)
    created_on = models.DateTimeField(_('Created On'), default=datetime.now, editable=False)
    updated_on = models.DateTimeField(_('Updated On'), editable=False, null=True, blank=True)

    class Meta:
        ordering = ['-created_on',]

    def __unicode__(self):
        return '%s' % (self.note)
    
    def save(self, *args, **kwargs):
        self.updated_on = datetime.now()
        super(Note, self).save()



from django.conf.urls.defaults import *
from django.core.urlresolvers import reverse


urlpatterns = patterns('loupe.views',
    url (r'^$', 
        view='dashboard',
        name='dashboard',
    ),

    url (r'^project/(?P<slug>[-_\w]+)/$', 
        view='project_detail',
        name='project_detail',
    ),

    url (r'^image/(\d+)/$', 
        view='image_detail',
        name='image_detail',
    ),

    url (r'^corkboard/(?P<slug>[-_\w]+)/$', 
        view='corkboard_detail',
        name='corkboard_detail',
    ),

    url(r'^corkboard/destroy/(\d+)/$',
        view='corkboard_destroy',
        name='corkboard_destroy'
    ),

    url(r'^image/destroy/(\d+)/$',
        view='image_destroy',
        name='image_destroy'
    ),
                       
    (r'^comments/', include('django.contrib.comments.urls')),
)



from django.contrib import admin
from django.db.models import get_model

admin.site.register(get_model('loupe', 'project'))
admin.site.register(get_model('loupe', 'image'))
admin.site.register(get_model('loupe', 'corkboard'))
admin.site.register(get_model('loupe', 'note'))



from django.template import Library, Node, Variable, TemplateSyntaxError
from django.core.urlresolvers import reverse
from loupe.models import Note, Image, Corkboard

register = Library()
 
class CorkNode(Node):
    def __init__(self, obj, varname):
        self.obj, self.varname = obj, varname

    def resolve(self, var, context):
        """Resolves a variable out of context if it's not in quotes"""
        if var[0] in ('"', "'") and var[-1] == var[0]:
            return var[1:-1]
        else:
            return Variable(var).resolve(context)

    def render(self, context):
        obj = self.resolve(self.obj, context)
        if self.varname:
            varname = self.resolve(self.varname, context)
        else:
            varname = 'corkboards'
        context[varname] = Corkboard.objects.filter(project=obj)
        return ''


@register.tag
def get_corkboards_for(parser, token):
    """
    returns corkboards for the given project object
    
    Example usage:
    {% get_corkboards_for obj as var %}

    """

    args = token.contents.split()
    if len(args) != 4:
        raise TemplateSyntaxError, "get_corkboards_for tag takes exactly 1 argument"
    if args[2] != 'as':
        raise TemplateSyntaxError, "third argument to the get_corkboards_for tag must be 'as'"

    return CorkNode(args[1], args[3])



from setuptools import setup, find_packages
 
setup(
    name='django-loupe',
    version='0.1.0',
    description='A simple collaborative image/screenshot review app for Django applications',
    long_description=open('README.rst').read(),
    author='Greg Newman',
    author_email='greg@20seven.org',
    url='http://bitbucket.org/gregnewman/django-loupe/',
    install_requires=(
        'django-imagekit>=0.3.2',
        'django-extensions>=0.4.1',
    ),
    packages=find_packages(exclude=['example']),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    zip_safe=False,
)



from datetime import datetime
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType

from loupe.models import Image, Corkboard, Note, Project

class ProjectForm(forms.ModelForm):

    class Meta:
        model = Project
        exclude = ('created_on', 'updated_on', 'user',)
    
    def __init__(self, user=None, *args, **kwargs):
        self.user = user
        super(ProjectForm, self).__init__(*args, **kwargs)

class CorkboardForm(forms.ModelForm):

    class Meta:
        model = Corkboard
        exclude = ('created_on', 'updated_on', 'user', 'images', 'project',)
    
    def __init__(self, user=None, *args, **kwargs):
        self.user = user
        super(CorkboardForm, self).__init__(*args, **kwargs)

class ImageForm(forms.ModelForm):

    class Meta:
        model = Image
        exclude = ('created_on', 'updated_on', 'user', 'num_views', 'corkboard',)
    
    def __init__(self, user=None, *args, **kwargs):
        self.user = user
        super(ImageForm, self).__init__(*args, **kwargs)

class NoteForm(forms.ModelForm):

    note = forms.CharField(label="Note")
    x1 = forms.IntegerField(widget=forms.HiddenInput)
    y1 = forms.IntegerField(widget=forms.HiddenInput)
    height = forms.IntegerField(widget=forms.HiddenInput)
    width = forms.IntegerField(widget=forms.HiddenInput)
    
    class Meta:
        model = Note
        exclude = ('created_on', 'updated_on', 'user', 'image',)
    
    def __init__(self, user=None, *args, **kwargs):
        self.user = user
        super(NoteForm, self).__init__(*args, **kwargs)



from django.shortcuts import render_to_response, get_object_or_404
from django.contrib.auth.decorators import login_required
from django.views.decorators.http import require_POST
from django.template import RequestContext
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.http import HttpResponse, HttpResponseForbidden

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext

from loupe.models import Project, Image, Corkboard, Note
from loupe.forms import ImageForm, CorkboardForm, NoteForm, ProjectForm

@login_required
def dashboard(request, template_name="loupe/dashboard.html"):
    """
    Displays the dashboard listing all of the corkboards and images
    that have been added as well as the form for adding new corkboards
    """
    projects = Project.objects.filter(members=request.user)
    form = ProjectForm()

    if request.method == "POST":
        form = ProjectForm(request.user, request.POST)
        if form.is_valid():
            project = form.save(commit=False)
            project.user = request.user
            project.save()
            form.save_m2m()
            request.user.message_set.create(message=_("Successfully created project '%s'") % project.title)
                
            return HttpResponseRedirect(reverse('project_detail', args=(project.slug,)))

    return render_to_response(template_name, {
        "projects": projects,
        "form": form,
    }, context_instance=RequestContext(request))

@login_required
def project_detail(request, slug, template_name="loupe/project_detail.html"):
    """
    Displays the corkboards for the given porject 'slug'
    to the user

    expects a slug and expects the user has access to the parent project
    """

    project = get_object_or_404(Project, slug=slug, members=request.user)
    if not request.user.is_staff:
        if request.user != project.members:
            request.user.message_set.create(message=_("You are trying to access a project you don't have permission to view!"))
            return HttpResponseRedirect(reverse('dashboard'))
     
    corkboards = Corkboard.objects.filter(project=project)
    form = CorkboardForm()
    
    if request.method == "POST":
        form = CorkboardForm(request.user, request.POST)
        if form.is_valid():
            corkboard = form.save(commit=False)
            corkboard.user = request.user
            corkboard.project = project
            corkboard.save()
            request.user.message_set.create(message=_("Successfully created corkboard '%s'") % corkboard.title)
                
            return HttpResponseRedirect(reverse('corkboard_detail', args=(corkboard.slug,)))
        
    return render_to_response(template_name, {
        "project": project,
        "form": form,
        "corkboards": corkboards,
    }, context_instance=RequestContext(request))

@login_required
def corkboard_detail(request, slug, template_name="loupe/corkboard_detail.html"):
    """
    Displays the corkboard and all it's associated images
    to the user

    expects a slug and expects the user has access to the parent project
    """

    corkboard = get_object_or_404(Corkboard, slug=slug)
    if not request.user.is_staff:
        if request.user != corkboard.project.members:
            request.user.message_set.create(message=_("You are trying to access a project you don't have permission to view!"))
            return HttpResponseRedirect(reverse('dashboard'))
        
    images = Image.objects.filter(corkboard=corkboard)

    form = ImageForm()
    
    if request.method == "POST":
        form = ImageForm(request.user, request.POST, request.FILES)
        if form.is_valid():
            image = form.save(commit=False)
            image.user = request.user
            image.corkboard = corkboard
            image.save()
            request.user.message_set.create(message=_("Successfully uploaded image '%s'") % image.title)
    
    return render_to_response(template_name, {
        "corkboard": corkboard,
        "form": form,
        "images": images,
    }, context_instance=RequestContext(request))

@login_required
def image_detail(request, id, template_name="loupe/image_detail.html"):
    """
    Displays the image to be discussed by the project members

    expects an id for the image and the user has access to the parent project
    """

    image = get_object_or_404(Image, pk=id)
    if not request.user.is_staff:
        if request.user != image.corkboard.project.members:
            request.user.message_set.create(message=_("You are trying to access a project you don't have permission to view!"))
            return HttpResponseRedirect(reverse('dashboard'))
    notes = Note.objects.filter(image=image)
    notes_count = notes.count()
    form = NoteForm()

    if request.method == "POST":
        form = NoteForm(request.user, request.POST)
        if form.is_valid():
            note = form.save(commit=False)
            note.user = request.user
            note.image = image
            note.save()
            request.user.message_set.create(message=_("Successfully attached note to image '%s'") % image.title)
                
    return render_to_response(template_name, {
        "image": image,
        "notes": notes,
        "notes_count": notes_count,
        "form": form,
    }, context_instance=RequestContext(request))


@require_POST
@login_required
def corkboard_destroy(request, id):
    """
    Deletes the corkboard passed in from the form
    """
    corkboard = get_object_or_404(Corkboard, pk=id)
    title = corkboard.title
    if not request.user.is_staff:
        return HttpResponseRedirect(reverse('dashboard'))

    corkboard.delete()
    request.user.message_set.create(message=_("Successfully deleted corkboard '%s'") % title)
    return HttpResponseRedirect(reverse('dashboard'))

@require_POST
@login_required
def image_destroy(request, id):
    """
    Deletes the corkboard passed in from the form
    """
    image = get_object_or_404(Image, pk=id)
    corkboard = image.corkboard
    title = image.title
    
    if not request.user.is_staff:
        return HttpResponseRedirect(reverse('dashboard'))

    image.delete()
    request.user.message_set.create(message=_("Successfully deleted image '%s'") % title)
    return HttpResponseRedirect(request.POST['next'])
